package simulation;

import configuration.SimulationConfiguration;
import configuration.SimulationConfigurationSingleton;
import metadata.BuildingMetadata;
import metadata.WarehouseMetadata;

import java.util.ArrayList;
import java.util.Collection;

public class Warehouse extends Building implements WarehouseSubject {
    private final Collection<Component> planes = new ArrayList<>();
    private final Collection<WarehouseObserver> observers = new ArrayList<>();

    public Warehouse(int id, String type, int x, int y) {
        super(id, type, x, y);
    }

    @Override
    public void processInput(Component input) {
        planes.add(input);
    }

    private void sellComponent() {
    }

    public void attach(WarehouseObserver observer) {
        observers.add(observer);
    }

    public void detach(WarehouseObserver observer) {
        observers.remove(observer);
    }

    public void notifyObservers() {
        boolean isFull = planes.size() >= getCapacity();
        observers.forEach(o -> o.update(isFull));
    }

    private static int getCapacity() {
        SimulationConfiguration configuration = SimulationConfigurationSingleton.getInstance().getConfiguration();
        WarehouseMetadata metadata = (WarehouseMetadata) configuration.getMetadata().get(BuildingMetadata.TYPE_WAREHOUSE);

        return metadata.getInput().getCapacity();
    }
}
