package view;

import configuration.SimulationConfiguration;
import configuration.SimulationConfigurationSingleton;
import configuration.SimulationData;
import metadata.BuildingMetadata;
import simulation.Building;
import simulation.BuildingState;
import simulation.Route;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.Serial;
import java.util.HashMap;
import java.util.Map;

public class MainPanel extends JPanel {
    @Serial
    private static final long serialVersionUID = 1L;

    private final Map<String, Image> icons = new HashMap<>();
    private final Map<Integer, Building> buildingsById = new HashMap<>();

    private SimulationConfiguration configuration;
    private SimulationData simulationData;

    public void reloadSimulation() {
        configuration = SimulationConfigurationSingleton.getInstance().getConfiguration();
        simulationData = configuration.getSimulationData();

        buildingsById.clear();
        simulationData.getBuildings().forEach(b -> buildingsById.put(b.getId(), b));

        icons.clear();
        configuration.getMetadata().values().forEach(m -> {
            try {
                loadImage(m);
            } catch (IOException e) {
                System.err.println("Could not load simulation icons");
            }
        });
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        // On ajoute à la position le delta x et y de la vitesse
//        position.translate(speed.x, speed.y);
//        g.fillRect(position.x, position.y, size, size);

        if (configuration == null) return;

        simulationData.getRoutes().forEach(r -> paintRoute(r, g));
        simulationData.getBuildings().forEach(b -> paintBuilding(b, g));
    }

    private void paintBuilding(Building building, Graphics g) {
        Image icon = icons.get(building.getType());
        g.drawImage(icon, building.getX(), building.getY(), null);
    }

    private void paintRoute(Route route, Graphics g) {
        Building fromBuilding = buildingsById.get(route.getFrom());
        Image fromIcon = icons.get(fromBuilding.getType());

        Building toBuilding = buildingsById.get(route.getTo());
        Image toIcon = icons.get(toBuilding.getType());

        g.drawLine(
                fromBuilding.getX() + fromIcon.getWidth(null) / 2,
                fromBuilding.getY() + fromIcon.getHeight(null) / 2,
                toBuilding.getX() + toIcon.getWidth(null) / 2,
                toBuilding.getY() + toIcon.getHeight(null) / 2
        );
    }

    private void loadImage(BuildingMetadata metadata) throws IOException {
        String iconPath = metadata.getIconsPaths().get(BuildingState.EMPTY);
        BufferedImage image = ImageIO.read(new File(iconPath));

        icons.put(metadata.getType(), image);
    }
}
